/*
* max7219.c
*
* Created: 22.10.2014 18:57:03
*  Author: sschultewolter
*/


#include "max7219.h"
#include "font.h"

// intern
void max7219_clock_out(uint8_t data)
{
	for(uint8_t i = 0; i < 8; i++)
	{
		// CLK auf LOW setzen
		MAX7219_PORT &= ~(1 << MAX7219_CLK);
		
		if(data & (1 << (7 - i))) MAX7219_PORT |= (1 << MAX7219_DIN);
		else MAX7219_PORT &= ~(1 << MAX7219_DIN);
		
		// CLK auf HIGH setzen, Daten uebernehmen
		MAX7219_PORT |= (1 << MAX7219_CLK);
	}
}

void max7219_send(uint8_t max7219_num_ic, uint8_t reg, uint8_t data)
{
	uint8_t i = 0;
	if(max7219_num_ic < MAX7219_NUM_IC)
	{
		// LOAD auf LOW setzen
		MAX7219_PORT &= ~(1 << MAX7219_LOAD);
		
		for(i = max7219_num_ic; i < (MAX7219_NUM_IC - 1); i++)
		{
			max7219_clock_out(MAX7219_NO_OP);
			max7219_clock_out(MAX7219_NO_OP);
		}
		
		max7219_clock_out(reg);
		max7219_clock_out(data);
		
		for(i = 0; i < max7219_num_ic; i++)
		{
			max7219_clock_out(MAX7219_NO_OP);
			max7219_clock_out(MAX7219_NO_OP);
		}
		
		// LOAD auf HIGH, steigende Flanke uebernimmt Daten in adressiertes Register
		MAX7219_PORT |= (1 << MAX7219_LOAD);
	}
}

// extern
void max7219_init(void)
{
	MAX7219_DDR |= (1 << MAX7219_CLK) | (1 << MAX7219_LOAD) | (1 << MAX7219_DIN);

	MAX7219_PORT |= (1 << MAX7219_LOAD);
	MAX7219_PORT &= ~((1<< MAX7219_CLK) | (1 << MAX7219_DIN));
	
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++)
	{
		max7219_send(ic, MAX7219_DEC_MODE, 0);
		max7219_send(ic, MAX7219_INTENSITY, 7);
		max7219_send(ic, MAX7219_SCAN_LIMIT, 7);
		max7219_send(ic, MAX7219_DISP_TEST, 0);
	}
	
	max7219_clear();
	max7219_on();
}

void max7219_intensity(uint8_t intensity)
{
	if(intensity > 15) return;
	
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_INTENSITY, intensity);
}

void max7219_disp_test(uint8_t disp_test)
{
	if(disp_test > 1) return;
	
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_DISP_TEST, disp_test);
}

void max7219_off(void)
{
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_SHUTDOWN, 0);
}

void max7219_on(void)
{
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++) max7219_send(ic, MAX7219_SHUTDOWN, 1);
}

void max7219_clear(void)
{
	for(uint8_t ic = 0; ic < MAX7219_NUM_IC; ic++)
	{
		for(uint8_t i = 0; i < 8; i++) max7219_send(ic, i + 1, 0);
	}
}
/*
void max7219_puts(char *s)
{
	
}*/

void max7219_puts(char *s, uint8_t bold, uint8_t invert, uint8_t scroll)
{

/*
	uint8_t size_text = strlen(s);
	
	uint8_t width = 0;
	
	if(bold) width = 8;
	else width = 5;
	
	uint16_t i = 0;
	static uint16_t j = 0;	
	while(i < (MAX7219_NUM_IC * 8))
	{
		uint8_t x = s[((i + j) / width) % size_text];
		uint8_t y = ((i + j) % width);
		uint8_t data = 0;
		if(bold) data = pgm_read_byte(&font_bold[x][y]);
		else data = pgm_read_byte(&font[x][y]);
		if(invert)max7219_send(i >> 3, ((i % 8) + 1), 255 - data);
		else max7219_send(i >> 3, ((i % 8) + 1), data);
		
		i++;
	}
	if(scroll)	if(++j > (size_text * width)-1) j = 0;*/

uint8_t size_text = strlen(s);

uint8_t width = 0;

if(bold) width = 8;
else width = 5;

uint16_t i = 0;
static uint16_t j = 0;
while(i < (MAX7219_NUM_IC * 8))
{
	uint8_t x = s[((i + j) / width) % size_text];
	uint8_t y = ((i + j) % width);
	uint8_t data = 0;
	if(bold) data = pgm_read_byte(&font_bold[x][y]);
	else data = pgm_read_byte(&font[x][y]);
	if(invert)max7219_send(i >> 3, ((i % 8) + 1), 255 - data);
	else max7219_send(i >> 3, ((i % 8) + 1), data);
	
	i++;
}
if(scroll)	if(++j > (size_text * width)-1) j = 0;
}

void max7219_clear_one(uint8_t ic)
{
	for(uint8_t i = 0; i < 8; i++) max7219_send(ic, i + 1, 0);
}

void max7219_putc(uint8_t ic, char c)
{

}